:: *****************************************************************************
:: *
:: * E M B E D D E D   W I Z A R D   P R O J E C T
:: *
:: *****************************************************************************
:: * FlashDownload.cmd
:: *
:: * This script uses the GD-Link Programmer to programm the internal and
:: * optionally also the external flash memory of a GD32 device.
:: *
:: * Usage:    FlashDownload.cmd <bin-file or elf-file> [options]
:: *
:: * Options:
:: *   separateConsole          -  open a separate console window
:: *   eraseFlash               -  erase the complete flash memory before programming
:: *   imageStartAddress=<addr> -  defines the start address a binary file has
:: *                               to be programmed to
:: *   reset                    -  reset target after flashing
:: *
:: *****************************************************************************


@ECHO OFF
setlocal ENABLEDELAYEDEXPANSION

:: initialize variables
set externalLoaderExtension=
set imageStartAddress=0x08000000
set objcopy=..\..\ThirdParty\gcc-arm-none-eabi\bin\arm-none-eabi-objcopy.exe
set FlashDownloadPath=%~dp0

echo.

:: Load commandline options
call :getCommadlineOptions "%*"

echo 'FlashDownload.cmd %1 %2 %3 %4 %5' called

:: display usage, if there is no parameter
if [%1] == [] (
  echo FlashDownload.cmd
  echo.
  echo This script uses the GD-Link Programmer to programm the internal and
  echo optionally also the external flash memory of a GD32 device.
  echo.
  echo Usage:    FlashDownload.cmd [elf-file or bin-file] [options]
  echo.
  echo Options:
  echo   separateConsole          -  open a separate console window
  echo   eraseFlash               -  erase the complete flash memory before programming
  echo   imageStartAddress=[addr] -  defines the start address a binary file has
  echo.                              to be programmed to
  echo   reset                    -  reset target after flashing
  echo.
  call pause
  exit
)

:: change size of console
if [%separateConsole%] == [set] (
  title Programming taget using GD-Link Programmer ...
  mode con:cols=140 lines=50
)

:: load configuration
for /F "delims= eol=:" %%A in (%~dps0FlashDownload.cfg) do set "%%A"


:: do some checks

if not exist "%FlashDownloadPath%\%GD32LinkProgrammerPath%" (
  echo.
  echo GD-Link Programmer check failed:
  echo GD-Link Programmer not found on path:
  echo "%FlashDownloadPath%\%GD32LinkProgrammerPath%".
  echo Please install the GD-Link Programmer or adapt the setting for
  echo 'GD32LinkProgrammerPath' in file '\Application\FlashDownload\GD-Link-Programmer.cfg'.
  echo.
  echo GD-Link Programmer can be downloaded at:
  echo https://www.dontknow.html
  echo.
  call :doPause
  exit /B 0
)


if [%~x1] == [.bin] goto startFlashing

if [%~x1] == [.out] goto startFlashing

:: convert input file, if it is no bin file
if [%~x1] == [.elf] (
  %FlashDownloadPath%\%objcopy% -O binary %1 %~p1%~n1.bin
  goto startFlashing
)


echo.
echo ERROR: input file type %~x1 not supported!
pause
exit /B 0


:startFlashing
set GD32LinkProgrammer="%FlashDownloadPath%\%GD32LinkProgrammerPath%\GD_Link_CLI.exe" %GD32LinkProgrammerParams%


echo connect > cmd.gdlink
echo r >> cmd.gdlink

if [%eraseFlash%] == [set] (
  echo erasing internal flash ...
  echo erase >> cmd.gdlink
)

echo load %~p1%~n1.bin %imageStartAddress% >> cmd.gdlink
echo r >> cmd.gdlink

echo Programming file '%~p1%~n1.bin' to target using GD-Link Programmer ...
%GD32LinkProgrammer% -commandfile cmd.gdlink

echo.

if %ERRORLEVEL% == 0 (
  echo '%~nx1' successfully flashed !!
  if [%separateConsole%] == [set] (
    timeout /t 3 > NUL
  )
) ELSE (
  echo ERROR - flashing failed!
  echo.
  call :doPause
)
exit /B 0



:getCommadlineOptions
  :: replace equal signs by ':'
  for /f "tokens=1-10 delims==" %%a in (%1) do set list3=%%a:%%b:%%c:%%d:%%e:%%f:%%g:%%h:%%i:%%j

  for %%i in (%list3%) do (
    set var=%%i
    call :setvar !var::= !
  )
  exit /B 0

:setvar
  if x%2==x (
    set %1=set
  ) else (
    set %1=%2
  )
  exit /B 0

:doPause
  if [%separateConsole%] == [set] (
    pause
  )
  exit /B 0